#################### step 2.整理ICGC临床数据 ####################
rm(list = ls())
options(stringsAsFactors = F)
library(readr)
library(dplyr)
load("exprSet.rda")
exprSet <- data.frame(exp_matrix,check.names = F)
test <- exprSet[1:10,1:10]
rownames(exprSet) <- exprSet[,1];exprSet <- exprSet[,-1]
exprSet <- t(exprSet)
test <- exprSet[1:10,1:10]
exprSet <- log2(exprSet+1)

donor_file <- read_delim(file="donor.LIRI-JP.tsv",delim="\t",col_names = TRUE)
donor_file <- donor_file %>%
  select(icgc_donor_id,donor_vital_status,donor_survival_time)
length(unique(donor_file$icgc_donor_id))
table(donor_file$donor_vital_status)

library(stringr)
colnames(exprSet) <- str_split(colnames(exprSet),"\\-",simplify = T)[,2]
table(colnames(exprSet) %in% donor_file$icgc_donor_id)

comSample <- intersect(colnames(exprSet),donor_file$icgc_donor_id)
expr <- data.frame(exprSet[,comSample],check.names = F)
clin <- donor_file[match(comSample,donor_file$icgc_donor_id),]

clin_expr <- data.frame(futime = clin$donor_survival_time,fustat = clin$donor_vital_status,t(expr))
test <- clin_expr[1:10,1:10]
table(clin_expr$fustat)
clin_expr$fustat <- ifelse(clin_expr$fustat=="alive",0,1)
table(clin_expr$fustat)
clin_expr_icgc <- clin_expr
save(clin_expr_icgc,file = "clin_expr_icgc.rda")



#################### step 3.训练集建模 ####################
rm(list = ls())
options(stringsAsFactors = F)
library(survival)
library(rms)

survival <- read.table("survival.tcga.txt",header = T,sep = "\t")
exprSet <- read.table("expression.tcga.txt",header = T,sep = "\t",row.names = 1,check.names = F)
exprSet <- exprSet[,substr(colnames(exprSet),14,15)=="01"]

exprSet <- data.frame(scale(t(exprSet)),check.names = F)
clin_expr_tcga <- merge(survival,exprSet,by.x = 1,by.y = 0)
rownames(clin_expr_tcga) <- clin_expr_tcga[,1];clin_expr_tcga <- clin_expr_tcga[,-1]

train <- clin_expr_tcga
fit <- coxph(Surv(futime,fustat)~.,data = train)
multiCoxSum <- summary(fit)


myCoefs <- as.numeric(multiCoxSum$coefficients[,"coef"])
cox_fea <- rownames(multiCoxSum$coefficients)
rs.train <- apply(train[,cox_fea], 1, function(x) {x %*% myCoefs})

#### 训练集森林图
library(survminer)
ggforest(fit, #Cox回归结果
         data = train,
         main = "Hazard ratio", #标题
         cpositions = c(0.05, 0.15, 0.35),  #前三列距离
         fontsize = 0.8, #字体大小
         refLabel = "reference", #相对变量的数值标签，也可改为1
         noDigits = 3) #保留HR值以及95%CI的小数位数

library(survminer)
pdf("forest_plot_train.pdf",width = 6,height = 7)
ggforest(fit, #Cox回归结果
         data = train,
         main = "Hazard ratio", #标题
         cpositions = c(0.05, 0.15, 0.35),  #前三列距离
         fontsize = 0.8, #字体大小
         refLabel = "reference", #相对变量的数值标签，也可改为1
         noDigits = 3) #保留HR值以及95%CI的小数位数
dev.off()



#################### step 4.测试集验证 ####################
load("clin_expr_icgc.rda")
(comGene <- intersect(colnames(clin_expr_tcga),colnames(clin_expr_icgc)))
# 交集基因只有8个，有一个基因在测试集没有。
# 训练集保持不变，测试集使用8个基因预测。
test <- clin_expr_icgc[,comGene]

test[,3:ncol(test)] <- scale(test[,3:ncol(test)])


myCoefs2 <- myCoefs[-3]
cox_fea2 <- cox_fea[-3]
rs.test <- apply(test[,cox_fea2], 1, function(x) {x %*% myCoefs2})

boxplot(train$futime/365)
boxplot(test$futime/365)
## 绘制1年、3年生存率的ROC曲线



#################### step 5.K-M曲线绘制 ####################
library(survival)
library(survminer)
train$riskScore <- rs.train
test$riskScore <- rs.test

res.cut <- surv_cutpoint(train, time = "futime",
                         event = "fustat", 
                         variables = "riskScore",
                         minprop = 0.3)
res.cut <- res.cut$cutpoint[[1]]

#### (1) 训练集K-M曲线绘制
dat <- train
dat$risk <- ifelse(dat$riskScore>res.cut,"high","low")

#画一幅高水准的生存曲线
library(survival)
library(survminer)
fit <- survfit(Surv(futime, fustat) ~ risk, data = dat)
ggsurvplot(fit,
           pval = TRUE, # 在图上添加log rank检验的p值
           conf.int = T,# 添加置信区间
           conf.int.style = "ribbon",  # 设置置信区间的风格
           risk.table.col = "strata", # 根据数据分组为风险表添加颜色
           ncensor.plot = TRUE,      # 展示随访过程中不同时间点死亡和删失的情况
           linetype = "strata", # 改变不同组别的生存曲线的线型
           surv.median.line = "hv", # 标注出中位生存时间
           ggtheme = theme_bw(), # 改变图形风格
           palette = c("#E7B800", "#2E9FDF")) # 图形颜色风格

ggsurvplot(fit,                    
           pval = FALSE,             
           conf.int = F, 
           fun = "cumhaz",
           conf.int.style = "ribbon",  # 设置置信区间的风格
           xlab = "Time in days",   # 设置x轴标签
           break.time.by = 180,     # 将x轴按照200为间隔进行切分
           ggtheme = theme_light(), # 设置图形风格
           risk.table = "abs_pct",  # 在风险表中添加绝对数和相对数
           risk.table.y.text.col = TRUE,# 设置风险表的文字颜色
           risk.table.y.text = FALSE,# 以条柱展示风险表的标签，而非文字
           ncensor.plot = TRUE,      # 展示随访过程中不同时间点死亡和删失的情况
           #surv.median.line = "hv", # 添加中位生存时间
           legend.labs = 
             c("Male", "Female"),    # 改变图例标签
           palette = 
             c("#E7B800", "#2E9FDF") # 设置颜色
)

ggsurvplot(fit,
           conf.int = TRUE,
           risk.table.col = "strata", 
           ggtheme = theme_bw(), 
           palette = c("#E7B800", "#2E9FDF"),
           fun = "cumhaz")


dat$futime <- dat$futime/365
fit <- survfit(Surv(futime, fustat) ~ risk, data = dat)
ggsurvplot(fit,
           pval = TRUE, # 在图上添加log rank检验的p值
           conf.int = T,# 添加置信区间
           conf.int.style = "step",  # 设置置信区间的风格
           xlab = "Time in years",   # 设置x轴标签
           break.time.by = 1,     # 将x轴按照1为间隔进行切分
           risk.table = TRUE, # 在图下方添加风险表
           risk.table.col = "strata", # 根据数据分组为风险表添加颜色
           ncensor.plot = TRUE,      # 展示随访过程中不同时间点死亡和删失的情况
           #linetype = "strata", # 改变不同组别的生存曲线的线型
           surv.median.line = "hv", # 标注出中位生存时间
           ggtheme = theme_bw(), # 改变图形风格
           palette = c("red","blue")) # 图形颜色风格


pdf("ggsurvplot.train.pdf",width = 5,height = 6)
ggsurvplot(fit,
           pval = TRUE, # 在图上添加log rank检验的p值
           conf.int = T,# 添加置信区间
           conf.int.style = "step",  # 设置置信区间的风格
           xlab = "Time in years",   # 设置x轴标签
           break.time.by = 1,     # 将x轴按照1为间隔进行切分
           risk.table = TRUE, # 在图下方添加风险表
           risk.table.col = "strata", # 根据数据分组为风险表添加颜色
           ncensor.plot = TRUE,      # 展示随访过程中不同时间点死亡和删失的情况
           #linetype = "strata", # 改变不同组别的生存曲线的线型
           surv.median.line = "hv", # 标注出中位生存时间
           ggtheme = theme_bw(), # 改变图形风格
           palette = c("red","blue")) # 图形颜色风格
dev.off()

#### (2) 测试集K-M曲线绘制
dat <- test
dat$risk <- ifelse(dat$riskScore>res.cut,"high","low")

#画一幅高水准的生存曲线
library(survival)
library(survminer)
fit <- survfit(Surv(futime, fustat) ~ risk, data = dat)
ggsurvplot(fit,
           pval = TRUE, # 在图上添加log rank检验的p值
           conf.int = T,# 添加置信区间
           conf.int.style = "ribbon",  # 设置置信区间的风格
           risk.table.col = "strata", # 根据数据分组为风险表添加颜色
           ncensor.plot = TRUE,      # 展示随访过程中不同时间点死亡和删失的情况
           linetype = "strata", # 改变不同组别的生存曲线的线型
           surv.median.line = "hv", # 标注出中位生存时间
           ggtheme = theme_bw(), # 改变图形风格
           palette = c("#E7B800", "#2E9FDF")) # 图形颜色风格

ggsurvplot(fit,                    
           pval = FALSE,             
           conf.int = F, 
           fun = "cumhaz",
           conf.int.style = "ribbon",  # 设置置信区间的风格
           xlab = "Time in days",   # 设置x轴标签
           break.time.by = 180,     # 将x轴按照200为间隔进行切分
           ggtheme = theme_light(), # 设置图形风格
           risk.table = "abs_pct",  # 在风险表中添加绝对数和相对数
           risk.table.y.text.col = TRUE,# 设置风险表的文字颜色
           risk.table.y.text = FALSE,# 以条柱展示风险表的标签，而非文字
           ncensor.plot = TRUE,      # 展示随访过程中不同时间点死亡和删失的情况
           #surv.median.line = "hv", # 添加中位生存时间
           legend.labs = 
             c("Male", "Female"),    # 改变图例标签
           palette = 
             c("#E7B800", "#2E9FDF") # 设置颜色
)

ggsurvplot(fit,
           conf.int = TRUE,
           risk.table.col = "strata", 
           ggtheme = theme_bw(), 
           palette = c("#E7B800", "#2E9FDF"),
           fun = "cumhaz")


dat$futime <- dat$futime/365
fit <- survfit(Surv(futime, fustat) ~ risk, data = dat)
ggsurvplot(fit,
           pval = TRUE, # 在图上添加log rank检验的p值
           conf.int = T,# 添加置信区间
           conf.int.style = "step",  # 设置置信区间的风格
           xlab = "Time in years",   # 设置x轴标签
           break.time.by = 1,     # 将x轴按照1为间隔进行切分
           risk.table = TRUE, # 在图下方添加风险表
           risk.table.col = "strata", # 根据数据分组为风险表添加颜色
           ncensor.plot = TRUE,      # 展示随访过程中不同时间点死亡和删失的情况
           #linetype = "strata", # 改变不同组别的生存曲线的线型
           surv.median.line = "hv", # 标注出中位生存时间
           ggtheme = theme_bw(), # 改变图形风格
           palette = c("red","blue")) # 图形颜色风格


pdf("ggsurvplot.test.pdf",width = 5,height = 6)
ggsurvplot(fit,
           pval = TRUE, # 在图上添加log rank检验的p值
           conf.int = T,# 添加置信区间
           conf.int.style = "step",  # 设置置信区间的风格
           xlab = "Time in years",   # 设置x轴标签
           break.time.by = 1,     # 将x轴按照1为间隔进行切分
           risk.table = TRUE, # 在图下方添加风险表
           risk.table.col = "strata", # 根据数据分组为风险表添加颜色
           ncensor.plot = TRUE,      # 展示随访过程中不同时间点死亡和删失的情况
           #linetype = "strata", # 改变不同组别的生存曲线的线型
           surv.median.line = "hv", # 标注出中位生存时间
           ggtheme = theme_bw(), # 改变图形风格
           palette = c("red","blue")) # 图形颜色风格
dev.off()


#################### step 6.riskplot绘制 ####################
#### (1) 训练集riskplot绘制
data <- train[,-ncol(train)]
if(!require(ggrisk)) install.packages('ggrisk')
if(!require(rms)) install.packages("rms")
if(!require(readxl)) install.packages("readxl")

library(survival)
library(ggrisk)
library(rms)
library(readxl)

fit <- coxph(Surv(futime,fustat)~.,data = data)

riskplot <- ggrisk(fit,#模型名称
                   cutoff.x = 30, #cutoff标签位置
                   cutoff.y = 2.5, #cutoff标签位置
                   #cutoff.value='median',
                   cutoff.value = res.cut) 
ggsave(riskplot,filename = "riskplot_train.pdf",width = 8,height = 8)


#### (2) 测试集riskplot绘制
data <- test[,-ncol(test)]
if(!require(ggrisk)) install.packages('ggrisk')
if(!require(rms)) install.packages("rms")
if(!require(readxl)) install.packages("readxl")

library(survival)
library(ggrisk)
library(rms)
library(readxl)

fit <- coxph(Surv(futime,fustat)~.,data = data)

riskplot <- ggrisk(fit,#模型名称
                   cutoff.x = 100, #cutoff标签位置
                   cutoff.y = 2, #cutoff标签位置
                   #cutoff.value='median',
                   cutoff.value = res.cut) 
ggsave(riskplot,filename = "riskplot_test.pdf",width = 8,height = 8)


#### 测试集森林图
library(survminer)
ggforest(fit, #Cox回归结果
         data = data,
         main = "Hazard ratio", #标题
         cpositions = c(0.05, 0.15, 0.35),  #前三列距离
         fontsize = 0.8, #字体大小
         refLabel = "reference", #相对变量的数值标签，也可改为1
         noDigits = 3) #保留HR值以及95%CI的小数位数

library(survminer)
pdf("forest_plot_test.pdf",width = 6,height = 7)
ggforest(fit, #Cox回归结果
         data = data,
         main = "Hazard ratio", #标题
         cpositions = c(0.05, 0.15, 0.35),  #前三列距离
         fontsize = 0.8, #字体大小
         refLabel = "reference", #相对变量的数值标签，也可改为1
         noDigits = 3) #保留HR值以及95%CI的小数位数
dev.off()



#################### step 7.ROC曲线绘制 ####################
library(timeROC)
#### (1)训练集ROC曲线绘制
data <- train
ROC_rt<-timeROC(T=data$futime/365,delta=data$fustat,
                marker=data$riskScore,cause=1,
                weighting="marginal",
                times=c(1,2,3,4,5),ROC=TRUE)
ROC_rt

plot(ROC_rt,time=1,col='#8F7700FF',title=FALSE,lwd=2)
plot(ROC_rt,time=2,col="#0073C2FF",add=TRUE,title=FALSE,lwd=2)
plot(ROC_rt,time=3,col="orange",add=TRUE,title=FALSE,lwd=2)
plot(ROC_rt,time=4,col="green",add=TRUE,title=FALSE,lwd=2)
plot(ROC_rt,time=5,col="#EFC000FF",add=TRUE,title=FALSE,lwd=2)
legend("bottomright",
       c(paste0("AUC at 1 years: ",round(ROC_rt$AUC[1],2)),
         paste0("AUC at 2 years: ",round(ROC_rt$AUC[2],2)),
         paste0("AUC at 3 years: ",round(ROC_rt$AUC[3],2)),
         paste0("AUC at 4 years: ",round(ROC_rt$AUC[4],2)),
         paste0("AUC at 5 years: ",round(ROC_rt$AUC[5],2))),
       col=c('#8F7700FF',"#0073C2FF","orange","green","#EFC000FF"),lwd=2,bty = "n")

pdf("ROC_train.pdf",width = 5,height = 5)
plot(ROC_rt,time=1,col='#8F7700FF',title=FALSE,lwd=2)
plot(ROC_rt,time=2,col="#0073C2FF",add=TRUE,title=FALSE,lwd=2)
plot(ROC_rt,time=3,col="orange",add=TRUE,title=FALSE,lwd=2)
plot(ROC_rt,time=4,col="green",add=TRUE,title=FALSE,lwd=2)
plot(ROC_rt,time=5,col="#EFC000FF",add=TRUE,title=FALSE,lwd=2)
legend("bottomright",
       c(paste0("AUC at 1 years: ",round(ROC_rt$AUC[1],2)),
         paste0("AUC at 2 years: ",round(ROC_rt$AUC[2],2)),
         paste0("AUC at 3 years: ",round(ROC_rt$AUC[3],2)),
         paste0("AUC at 4 years: ",round(ROC_rt$AUC[4],2)),
         paste0("AUC at 5 years: ",round(ROC_rt$AUC[5],2))),
       col=c('#8F7700FF',"#0073C2FF","orange","green","#EFC000FF"),lwd=2,bty = "n")
dev.off()


#### (2)测试集ROC曲线绘制
data <- test
ROC_rt<-timeROC(T=data$futime/365,delta=data$fustat,
                marker=data$riskScore,cause=1,
                weighting="marginal",
                times=c(1,2,3,4,5),ROC=TRUE)
ROC_rt

plot(ROC_rt,time=2,col='#8F7700FF',title=FALSE,lwd=2)
plot(ROC_rt,time=3,col="#0073C2FF",add=TRUE,title=FALSE,lwd=2)
plot(ROC_rt,time=4,col="#EFC000FF",add=TRUE,title=FALSE,lwd=2)
legend("bottomright",
       c(paste0("AUC at 2 years: ",round(ROC_rt$AUC[2],2)),
         paste0("AUC at 3 years: ",round(ROC_rt$AUC[3],2)),
         paste0("AUC at 4 years: ",round(ROC_rt$AUC[4],2))),
       col=c('#8F7700FF',"#0073C2FF","#EFC000FF"),lwd=2,bty = "n")

pdf("ROC_test.pdf",width = 5,height = 5)
plot(ROC_rt,time=2,col='#8F7700FF',title=FALSE,lwd=2)
plot(ROC_rt,time=3,col="#0073C2FF",add=TRUE,title=FALSE,lwd=2)
plot(ROC_rt,time=4,col="#EFC000FF",add=TRUE,title=FALSE,lwd=2)
legend("bottomright",
       c(paste0("AUC at 2 years: ",round(ROC_rt$AUC[2],2)),
         paste0("AUC at 3 years: ",round(ROC_rt$AUC[3],2)),
         paste0("AUC at 4 years: ",round(ROC_rt$AUC[4],2))),
       col=c('#8F7700FF',"#0073C2FF","#EFC000FF"),lwd=2,bty = "n")
dev.off()

